/* Structures to use in vortex2d */

#ifndef structs_included
#define structs_included 1


/* A vector is simply a 2-ple, and can be used to store
 * pretty much anything
 */
typedef struct two_d_vector {
   double x[2];\
} VEC;


/* This structure defines the space upon which to solve
 * the two-dimensional vortex flow upon
 */
typedef struct solution_space {
   VEC o;	/* origin */
   VEC s;	/* size, in x and y */
} SPACE;


/* An element is a fluid dynamic signularity, "type" is 'v'
 * for a vortex, 's' for a source/sink,  and 'f' for a 
 * freestream, there are no doublets. Also, 'b' is used
 * for a vortex blob, one with a viscous core and a 
 * characteristic radius squared (stored as val)
 */
typedef struct element_record {
   int index;	/* on which object is it; -1 if free, 0 if ground */
   char type;	/* type of singularity */
   VEC loc;	/* element's absolute location if free, 
                 * relative to unrotated cm of object if not 
                 * OR, the velocity vector if type='f' */
   double str;	/* singularity strength */
   double val;	/* optional second value, angle or momentum radius */
} ELEMENT;


/* A generator is a point in space, usually fixed to ground
 * (object 0) or a free object, and creates singularities,
 * depending on type and surrounding conditions
 * The model.str value should be the strength per unit
 * time created by the generator
 */
typedef struct element_generator_record {
   ELEMENT model;	/* defines type of element to generate */
   VEC loc;
   double rate;  	/* the flow rate in elements per second */
} ELEMENT_GENERATOR;


/* A particle is a graphic element that does not rotate, and
 * is assigned a specific VEC in space
 */
typedef struct a_particle {
   VEC loc;	/* the particle's location */
   double age;	/* the particle's lifespan */
} PARTICLE_OLD;


/* This is an attempt at creating a linked list
 */
typedef struct particle_record *particle_pointer;
typedef struct particle_record {
   VEC loc;     /* the particle's location */
   double off_time; /* the particle's lifespan */
   particle_pointer next_particle; /* pointer to the next particle in the list */
} PARTICLE;


/* A particle generator emits particles around a primary
 * direction vector. The actual speed and direction of
 * each particle emitted depends on the flowfield and 
 * a random number
 */
typedef struct particle_generator_record {
   VEC loc;	/* the location of the particle generator */
   double rad;	/* the radius within which to create particles */
   double rate;	/* the flow rate in particles per second */
   double age_low;
   double age_high;
                /* the age bounds for the particles generated */
} PARTICLE_GENERATOR;


/* An object contains pointers to a list of nodes that define
 * the local coordinates of the boundary of the object. An
 * object can have a different speed than the flow around it.
 */
typedef struct solid_object {
   int isFixed;	/* is the object fixed to ground? */
   VEC loc;	/* center of mass */
   VEC vel;	/* velocity, x and y */
   VEC rot;	/* rotational position and velocity */
   double I;	/* inertia coefficient, translational */
   double J;	/* coefficient of rotational inertia */
   VEC node[MAX_OBJECT_NODES];
		/* list of nodes making up the object */
		/* the collocation points and vortex locations
		 * are functions of these node's locations */
} OBJECT;

#endif
/* end */
